package com.lframework.xingyun.sc.bo.sale;

import com.alibaba.excel.annotation.format.DateTimeFormat;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.lframework.starter.common.constants.StringPool;
import com.lframework.starter.common.utils.StringUtil;
import com.lframework.starter.web.bo.BaseBo;
import com.lframework.starter.web.common.utils.ApplicationUtil;
import com.lframework.starter.web.components.validation.IsEnum;
import com.lframework.starter.web.dto.BaseDto;
import com.lframework.xingyun.basedata.entity.Customer;
import com.lframework.xingyun.basedata.entity.StoreCenter;
import com.lframework.xingyun.basedata.service.customer.CustomerService;
import com.lframework.xingyun.basedata.service.storecenter.StoreCenterService;
import com.lframework.xingyun.sc.entity.SaleOrder;
import com.lframework.xingyun.sc.enums.SaleOrderIsLease;
import com.lframework.xingyun.sc.enums.SaleOrderPurchaseStatus;
import com.lframework.xingyun.template.core.service.UserService;
import io.swagger.annotations.ApiModelProperty;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

import lombok.Data;

@Data
public class QuerySaleOrderBo extends BaseBo<SaleOrder> implements BaseDto {

    /**
     * ID
     */
    @ApiModelProperty("ID")
    private String id;

    /**
     * 单号
     */
    @ApiModelProperty("单号")
    private String code;

    /**
     * 仓库编号
     */
    @ApiModelProperty("仓库编号")
    private String scCode;

    /**
     * 仓库名称
     */
    @ApiModelProperty("仓库名称")
    private String scName;

    /**
     * 客户编号
     */
    @ApiModelProperty("客户编号")
    private String customerCode;

    /**
     * 客户名称
     */
    @ApiModelProperty("客户名称")
    private String customerName;

    /**
     * 销售员姓名
     */
    @ApiModelProperty("销售员姓名")
    private String salerName;

    /**
     * 销售数量
     */
    @ApiModelProperty("销售数量")
    private Integer totalNum;

    /**
     * 赠品数量
     */
    @ApiModelProperty("赠品数量")
    private Integer totalGiftNum;

    /**
     * 销售金额
     */
    @ApiModelProperty("销售金额")
    private BigDecimal totalAmount;

    /**
     * 备注
     */
    @ApiModelProperty("备注")
    private String description;

    /**
     * 创建人
     */
    @ApiModelProperty("创建人")
    private String createBy;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
    @JsonFormat(pattern = StringPool.DATE_TIME_PATTERN)
    private LocalDateTime createTime;

    /**
     * 审核人
     */
    @ApiModelProperty("审核人")
    private String approveBy;

    /**
     * 审核时间
     */
    @ApiModelProperty("审核时间")
    @JsonFormat(pattern = StringPool.DATE_TIME_PATTERN)
    private LocalDateTime approveTime;

    /**
     * 状态
     */
    @ApiModelProperty("状态")
    private Integer status;

    /**
     * 拒绝原因
     */
    @ApiModelProperty("拒绝原因")
    private String refuseReason;

    /**
     * 本单利润
     */
    private BigDecimal thisOrderProfit;

    /**
     * 客户费用
     */
    private BigDecimal customMoney;

    /**
     * 采购价
     */
    private BigDecimal purchaseDecimal;

    /**
     * 采购总价
     */
    private BigDecimal totalPurchasePrice;

    /**
     * 运费
     */
    private BigDecimal shippingFee;

    /**
     * 是否租赁订单
     */
    private Integer isLease;


    /**
     * 采购状态，0未采购、2完成采购、3部分采购
     */
    //@IsEnum(message = "格式不正确！", enumClass = SaleOrderPurchaseStatus.class)
    private Integer purchaseStatus;

    /**
     * 租赁起始日期
     */
    @DateTimeFormat(StringPool.DATE_PATTERN)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime leaseStartTime;

    /**
     * 租赁结束日期
     */
    @DateTimeFormat(StringPool.DATE_PATTERN)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime leaseEndTime;


    @ApiModelProperty(value = "序列号")
    private String productName;

    /**
     * 毛利率 仅做计算，不做存储
     * 毛利率 = 本单利润/含税总金额* 100%
     */
    @ApiModelProperty("毛利率")
    private String grossMargin;

    public QuerySaleOrderBo(SaleOrder dto) {

        super(dto);
    }

    @Override
    public BaseBo<SaleOrder> convert(SaleOrder dto) {

        return super.convert(dto, QuerySaleOrderBo::getStatus);
    }

    @Override
    protected void afterInit(SaleOrder dto) {

    //    StoreCenterService storeCenterService = ApplicationUtil.getBean(StoreCenterService.class);
//        StoreCenter sc = storeCenterService.findById(dto.getScId());
//        this.scCode = sc.getCode();
//        this.scName = sc.getName();

        CustomerService customerService = ApplicationUtil.getBean(CustomerService.class);
        Customer customer = customerService.findById(dto.getCustomerId());
        this.customerCode = customer.getCode();
        this.customerName = customer.getName();

        UserService userService = ApplicationUtil.getBean(UserService.class);
        if (!StringUtil.isBlank(dto.getSalerId())) {
            this.salerName = userService.findById(dto.getSalerId()).getName();
        }

        if (!StringUtil.isBlank(dto.getApproveBy())) {
            this.approveBy = userService.findById(dto.getApproveBy()).getName();
        }

        this.status = dto.getStatus().getCode();
    }
}
